@@ -3,7 +3,8 @@
 from lxml import etree
 from Crypto.Hash import SHA256
 
-from .contants import TRASLADO, EGRESO, INGRESO, PAGO, COMPLEMENTO_CARTA_PORTE, COMPLEMENTO_PAGOS
+from .contants import TRASLADO, EGRESO, INGRESO, PAGO, COMPLEMENTO_CARTA_PORTE, COMPLEMENTO_PAGOS, ST_RECIBO_DONATIVO, \
+    COMPLEMENTO_DONATARIAS
 from .generator_abstract import AbstractGeneratorXml
 from .utils import replace
 
@@ -14,6 +15,11 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
         super(self.__class__, self).__init__(data)
         self.version = '3.3'
         self.algoritmoSha = SHA256
+        if self.data:
+            self.impuestos_trasladados = self.data.get('impuestos_trasladados', [])
+            self.impuestos_retenidos = self.data.get('impuestos_retenidos', [])
+            self.impuestos_trasladados_p = self.data.get('impuestos_trasladados_p', [])
+            self.impuestos_retenidos_p = self.data.get('impuestos_retenidos_p', [])
 
     def generate_xml(self):
         self.reset_dom()
@@ -22,7 +28,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
         # xmlns: xsi = "http://www.w3.org/2001/XMLSchema-instance"
         # xmlns: cartaporte = "http://www.sat.gob.mx/CartaPorte"
         # xsi: schemaLocation = "http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd http://www.sat.gob.mx/CartaPorte http://www.sat.gob.mx/sitio_internet/cfd/CartaPorte/CartaPorte.xsd"
-        if self.data.get('tipo_comprobante') == 4: #recibo donativo
+        if self.tipo_comprobante == INGRESO and self.data.get('subtipo') == ST_RECIBO_DONATIVO: #recibo donativo
             dom.setAttribute('xsi:schemaLocation',
                              'http://www.sat.gob.mx/cfd/3 ' +
                              'http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv33.xsd ' +
@@ -51,8 +57,9 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
         dom.setAttribute('Sello', '')
         dom.setAttribute('NoCertificado', self.data.get('no_certificado'))
         dom.setAttribute('Certificado','')
-        dom.setAttribute('TipoDeComprobante', self.tipo_objeto)
-        dom.setAttribute('LugarExpedicion', self.data.get('codigo_postal', ''))
+
+        dom.setAttribute('TipoDeComprobante', self.tipo_comprobante)
+        dom.setAttribute('LugarExpedicion', self.data.get('lugar_expedicion', ''))
         if self.data.get('confirmacion'):
             if len(self.data.get('confirmacion', [])) == 5:
                 dom.setAttribute('Confirmacion', self.data.get('confirmacion'))  # opcional
@@ -65,7 +72,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
         dom.appendChild(self.get_emisor())
         dom.appendChild(self.get_receptor())
 
-        if (self.tipo_objeto == INGRESO) or (self.tipo_objeto == EGRESO and not self.data.get('concepto_fijo')):
+        if (self.tipo_comprobante == INGRESO) or (self.tipo_comprobante == EGRESO and not self.data.get('concepto_fijo')):
             dom.setAttribute('FormaPago', self.data.get('forma_pago'))
             dom.setAttribute('CondicionesDePago', self.data.get('condicion_pago'))
             # dom.setAttribute('Descuento', '0.00') # 'Opcional'
@@ -84,12 +91,12 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 dom.appendChild(conceptos)
             if self.data.get('iva') != '0' and self.data.get('iva') != 0:
                 dom.appendChild(self.get_impuestos())
-        elif self.tipo_objeto == PAGO:
+        elif self.tipo_comprobante == PAGO:
             dom.setAttribute('SubTotal','0')
             dom.setAttribute('Moneda','XXX')
             dom.setAttribute('Total','0')
             dom.appendChild(self.get_concepto_fijo())
-        elif self.tipo_objeto == TRASLADO:
+        elif self.tipo_comprobante == TRASLADO:
             dom.setAttribute('SubTotal','0')
             dom.setAttribute('Moneda','XXX')
             dom.setAttribute('Total','0')
@@ -99,7 +106,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 for concepto in conceptos_list:
                     conceptos.appendChild(self.get_conceptos(concepto))
                 dom.appendChild(conceptos)
-        elif self.tipo_objeto == EGRESO and self.data.get('concepto_fijo'):
+        elif self.tipo_comprobante == EGRESO and self.data.get('concepto_fijo'):
             dom.setAttribute('FormaPago',self.data.get('forma_pago'))
             dom.setAttribute('CondicionesDePago', self.data.get('condicion_pago'))
             dom.setAttribute('MetodoPago', self.data.get('metodo_pago'))
@@ -110,7 +117,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
             dom.setAttribute('Total',str(self.data.get('total')))
             dom.appendChild(self.get_concepto_fijo())
             dom.appendChild(self.get_impuestos())
-        if self.data.get('tipo_comprobante') in [4, 5] or self.complemento in [COMPLEMENTO_PAGOS, COMPLEMENTO_CARTA_PORTE]:
+        if self.complemento in [COMPLEMENTO_DONATARIAS, COMPLEMENTO_PAGOS, COMPLEMENTO_CARTA_PORTE]:
             dom.appendChild(self.get_complementos())
         self.dom = dom
         return self.dom
@@ -146,7 +153,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
             if self.data.get('receptor_residencia_fiscal') != 'MEX':
                 receptor.setAttribute('ResidenciaFiscal',self.data.get('receptor_residencia_fiscal'))
                 receptor.setAttribute('NumRegIdTrib',self.data.get('receptor_num_reg_id_trib'))
-            if self.tipo_objeto == TRASLADO and self.data.get('uso_cfdi') != 'P01':
+            if self.tipo_comprobante == TRASLADO and self.data.get('uso_cfdi') != 'P01':
                 raise Exception('El campo Uso CFDI debe contener el valor P01 - Por Definir')
             receptor.setAttribute('UsoCFDI', self.data.get('uso_cfdi'))
             return receptor
@@ -160,11 +167,11 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
             concepto.setAttribute('ClaveProdServ','84111506')
             concepto.setAttribute('Cantidad','1')
             concepto.setAttribute('ClaveUnidad','ACT')
-            if self.tipo_objeto == PAGO:
+            if self.tipo_comprobante == PAGO:
                 concepto.setAttribute('Descripcion','Pago')
                 concepto.setAttribute('ValorUnitario','0')
                 concepto.setAttribute('Importe','0')
-            elif self.tipo_objeto == EGRESO:
+            elif self.tipo_comprobante == EGRESO:
                 if self.data.get('descripcion', '') != '':
                     concepto.setAttribute('Descripcion',replace(self.data.get('descripcion', '').strip()))
                 else:
@@ -177,46 +184,52 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
             raise Exception("Ocurrio un error. Error::: " + str(e))
 
     def get_impuestos(self):
-        # Agrega seccion de impuesto total, falta TotalImpuestosRetenidos
         try:
             impuestos = self.dom.createElement('cfdi:Impuestos')
-            if bool(self.data.get('aplica_retencion_iva', False)):
+            if len(self.impuestos_retenidos) > 0:
+                retenciones = self.dom.createElement('cfdi:Retenciones')
+                total_retenciones = 0
+                for retencion in self.impuestos_retenidos:
+                    total_retenciones += float(retencion.get('importe', 0))
+                    retenciones.appendChild(self.get_retenciones(retencion))
+                impuestos.appendChild(retenciones)
                 impuestos.setAttribute('TotalImpuestosRetenidos', str(self.data.get('retencion_iva')))
-                impuestos.appendChild(self.get_retenciones())
-            if bool(self.data.get('aplica_iva', False)):
+
+            if len(self.impuestos_trasladados) > 0:
+                traslados = self.dom.createElement('cfdi:Traslados')
+                total_traslados = 0
+                for traslado in self.impuestos_trasladados:
+                    total_traslados += float(traslado.get('importe', 0))
+                    impuestos.appendChild(self.get_traslados(traslado))
+                impuestos.appendChild(traslados)
                 impuestos.setAttribute('TotalImpuestosTrasladados',str(self.data.get('iva')))
-                impuestos.appendChild(self.get_traslados())
             return impuestos
         except Exception as e:
             raise Exception("Ocurrio un error. Error::: " + str(e))
 
-    def get_traslados(self):
-        traslados = self.dom.createElement('cfdi:Traslados')
+    def get_traslados(self, traslado_data):
         traslado = self.dom.createElement('cfdi:Traslado')
-        traslado.setAttribute('Impuesto', '002')
-        if self.data.get('iva') != '0' and self.data.get('iva') != 0:
-            traslado.setAttribute('TipoFactor', self.data.get('tipo_factor', 'Tasa'))
-            traslado.setAttribute('TasaOCuota', str(self.data.get('porcentaje')))
-            traslado.setAttribute('Importe', str(self.data.get('iva')))
-        else:
-            traslado.setAttribute('TipoFactor', 'Exento')
-        traslados.appendChild(traslado)
-        return traslados
+        traslado.setAttribute('Base', str(traslado_data.get('base')))
+        traslado.setAttribute('Impuesto', traslado_data.get('impuesto'))
+        traslado.setAttribute('TipoFactor', traslado_data.get('tipo_factor'))
+        if traslado_data.get('tasa_cuota'):
+            traslado.setAttribute('TasaOCuota', str(traslado_data.get('tasa_cuota')))
+        if traslado_data.get('importe'):
+            traslado.setAttribute('Importe', str(traslado_data.get('importe')))
+        return traslado
 
-    def get_retenciones(self):
-        retenciones = self.dom.createElement('cfdi:Retenciones')
+    def get_retenciones(self, retencion_data):
         retencion = self.dom.createElement('cfdi:Retencion')
-        retencion.setAttribute('Impuesto', '002')
-        retencion.setAttribute('Importe', str(self.data.get('retencion_iva')))
-        retenciones.appendChild(retencion)
-        return retenciones
+        retencion.setAttribute('Impuesto', retencion_data.get('impuesto'))
+        retencion.setAttribute('Importe', str(retencion_data.get('importe')))
+        return retencion
 
     def get_complementos(self):
         try:
             complemento = self.dom.createElement('cfdi:Complemento')
-            if self.data.get('tipo_comprobante') == 4:
+            if self.complemento == COMPLEMENTO_DONATARIAS:
                 complemento.appendChild(self.get_donatarias())
-            elif self.data.get('tipo_comprobante') == 5:
+            elif self.complemento == COMPLEMENTO_PAGOS:
                 complemento.appendChild(self.get_pagos())
             elif self.complemento == COMPLEMENTO_CARTA_PORTE:
                 complemento.appendChild(self.get_complemento_cartaporte())
@@ -316,7 +329,7 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
                 raise Exception('Se debe especificar una descripción para poder timbrar y posteriormente crear una poliza.')
             cfdi_concepto.setAttribute('ValorUnitario', str(concepto.get('valor_unitario')))
             cfdi_concepto.setAttribute('Importe', str(concepto.get('importe')))
-            if self.tipo_objeto != TRASLADO:
+            if self.tipo_comprobante != TRASLADO:
                 #revisar en el manual si lleva impuestos este tipo de comprobante
                 cfdi_concepto.appendChild(self.get_impuestos_concepto(concepto))
             if concepto.get('informacion_aduanera'):
@@ -329,37 +342,38 @@ class GeneratorXmlThreeDotThree(AbstractGeneratorXml):
 
     def get_impuestos_concepto(self,concepto):
         impuestos = self.dom.createElement('cfdi:Impuestos')
-        if bool(self.data.get('aplica_iva', False)):
-            impuestos.appendChild(self.get_traslados_concepto(concepto))
-        if bool(self.data.get('aplica_retencion_iva', False)):
-            impuestos.appendChild(self.get_retenciones_concepto(concepto))
+        if len(concepto.get('traslados', [])) > 0:
+            traslados = self.dom.createElement('cfdi:Traslados')
+            for traslado in concepto.get('traslados', []):
+                traslados.appendChild(self.get_traslados_concepto(traslado))
+            impuestos.appendChild(traslados)
+        if len(concepto.get('retenciones', [])) > 0:
+            retenciones = self.dom.createElement('cfdi:Retenciones')
+            for retencion in concepto.get('retenciones', []):
+                retenciones.appendChild(self.get_retenciones_concepto(retencion))
+            impuestos.appendChild(retenciones)
         return impuestos
 
-    def get_traslados_concepto(self, concepto):
-        traslados = self.dom.createElement('cfdi:Traslados')
+    def get_traslados_concepto(self, traslado_data):
         traslado = self.dom.createElement('cfdi:Traslado')
-        traslado.setAttribute('Impuesto','002')  # 'IVA'
-        if concepto.get('iva') != '0' and concepto.get('iva') != 0:
-            traslado.setAttribute('TipoFactor', concepto.get('tipo_factor', 'Tasa'))  # Tipo factor Tasa ó Cuota
-            traslado.setAttribute('Base', str(concepto.get('base')))  # Importe del concepto
-            traslado.setAttribute('TasaOCuota', str(concepto.get('porcentaje')))  # valor del impuesto que se envia
-            traslado.setAttribute('Importe', str(concepto.get('iva')))
-        else:
-            traslado.setAttribute('TipoFactor', 'Exento')
-            traslado.setAttribute('Base', str(concepto.get('importe')))
-        traslados.appendChild(traslado)
-        return traslados
+        traslado.setAttribute('Base', str(traslado_data.get('base')))  # Importe del concepto
+        traslado.setAttribute('Impuesto', traslado_data.get('impuesto', ''))
+        traslado.setAttribute('TipoFactor', traslado_data.get('tipo_factor', 'Tasa'))
+        if traslado_data.get('tipo_factor') != 'Exento':
+            traslado.setAttribute('TasaOCuota', str(traslado_data.get('tasa_cuota')))
+            traslado.setAttribute('Importe', str(traslado_data.get('importe')))
+        return traslado
 
-    def get_retenciones_concepto(self, concepto):
-        retenciones = self.dom.createElement('cfdi:Retenciones')
+    def get_retenciones_concepto(self, retencion_data):
         retencion = self.dom.createElement('cfdi:Retencion')
-        retencion.setAttribute('Impuesto', '002')  # 'IVA'
-        retencion.setAttribute('TipoFactor', concepto.get('retencion_tipo_factor', 'Tasa'))  # Tipo factor Tasa ó Cuota
-        retencion.setAttribute('Base', str(concepto.get('base')))  # Importe del concepto
-        retencion.setAttribute('TasaOCuota', str(concepto.get('retencion_porcentaje')))  # valor del impuesto que se envia
-        retencion.setAttribute('Importe', str(concepto.get('retencion_iva')))
-        retenciones.appendChild(retencion)
-        return retenciones
+        retencion.setAttribute('Base', str(retencion_data.get('base')))  # Importe del concepto
+        retencion.setAttribute('Impuesto', retencion_data.get('impuesto'))
+        retencion.setAttribute('TipoFactor', retencion_data.get('tipo_factor', 'Tasa'))  # Tipo factor Tasa ó Cuota
+        if retencion_data.get('tipo_factor') != 'Exento':
+            retencion.setAttribute('TasaOCuota',
+                                   str(retencion_data.get('tasa_cuota')))  # valor del impuesto que se envia
+            retencion.setAttribute('Importe', str(retencion_data.get('importe')))
+        return retencion
 
     def get_informacion_aduanera(self, informacion_aduanera):
         try:
